# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2002-2003 Juan David Ibez Palomar <jdavid@itaapy.com>
#               2003 Itaapy <contact@itaapy.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA


__revision__ = "$Id: catalog.py,v 1.11 2003/08/20 09:52:33 jdavid Exp $"


# Import Python modules
import sets
from types import ListType

# Import ZODB modules
try:
    import ZODB
    from Persistence import Persistent, PersistentMapping
except ImportError:
    # To run the unit tests
    class Persistent:
        pass

    from UserDict import UserDict as PersistentMapping

# Import itools modules
from itools.catalog.index import ValueIndex
from itools.upgrade import Upgradeable


class CatalogError(Exception): pass



class Catalog(Upgradeable, Persistent):
    """
    A catalog lets to index and search objects. I keeps a list of indexes..
    """

    __version__ = '20030804'


    def __init__(self):
        Upgradeable.__init__(self)

        self.indexes = PersistentMapping()


    #########################################################################
    # API
    #########################################################################
    def add_index(self, name, index=None):
        if index is None:
            index = ValueIndex()
        self.indexes[name] = index


    def remove_index(self, name):
        del self.indexes[name]


    def index_object(self, oid, object):
        """
        (Re-)indexes the given object.
        """
        for name, index in self.indexes.items():
            # First the object is unindexed
            index.remove(oid)

            # Get the value
            value = getattr(object, name, None)
            if value is not None:
                if callable(value):
                    value = value()

                # Indexes the object
                if type(value) is ListType:
                    apply(index.add, (oid,) + tuple(value))
                else:
                    index.add(oid, value)


    def unindex_object(self, oid):
        """
        Unindexes the given object.
        """
        for name, index in self.indexes.items():
            # First the object is unindexed
            index.remove(oid)


    def get_oids(self):
        """
        Returns all the indexed objects.
        """
        set = sets.Set()
        for index in self.indexes.values():
            set.union_update(sets.Set(index.reverse.keys()))
        return set


    def __getattr__(self, name):
        if self.indexes.has_key(name):
            return self.indexes[name]

        raise AttributeError, name


##    def search(self, **kw):
##        """
##        Search objects for the given criteria. Objects must match *all*
##        the criteria.
##        """
##        found = []
##        for key, value in kw.items():
##            index = self.indexes.get(key, None)
##            if index is None:
##                raise CatalogError, 'index "%s" does not exist' % key
##            found.append(index.search(value))

##        # Compact the objects (intersection)
##        nfound = len(found)
##        if nfound == 0:
##            return []
##        elif nfound == 1:
##            return found[0]

##        oids = []
##        for oid in found[0]:
##            for aux in found[1:]:
##                if oid not in aux:
##                    break
##            else:
##                oids.append(oid)

##        return oids
