# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2002-2003 Juan David Ibez Palomar <jdavid@itaapy.com>
#               2003 Itaapy <contact@itaapy.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA


__revision__ = '$Id: test_catalog.py,v 1.4 2003/07/31 16:19:45 jdavid Exp $'


from sets import Set
import unittest

from catalog import Catalog
from index import ValueIndex


class Software:
    def __init__(self, name, type, keywords):
        self.name = name
        self.type = type
        self.keywords = keywords


class CatalogTestCase(unittest.TestCase):
    def test_search(self):
        # Initialize the Catalog
        catalog = Catalog()
        catalog.add_index('type')
        catalog.add_index('keywords')

        # Initialize objects
        objects = [Software('python', 'language', ['easy', 'slow']),
                   Software('perl', 'language', ['hard', 'slow']),
                   Software('C', 'language', ['hard', 'fast']),
                   Software('linux', 'os', [])]

        # Index the objects
        for object in objects:
            catalog.index_object(object.name, object)
        
        found = catalog.type._is('language') & catalog.keywords._is('slow')
        expected = Set(['python', 'perl'])

        assert found == expected


    def test_reindex(self):
        catalog = Catalog()
        catalog.add_index('type')
        catalog.add_index('keywords')

        objects = [Software('python', 'language', ['easy', 'fast']),
                   Software('python', 'language', ['easy', 'slow'])]
        for object in objects:
            catalog.index_object(object.name, object)

        assert catalog.keywords._is('slow') == Set(['python'])
        assert catalog.keywords._is('fast') == Set([])



if __name__ == '__main__':
    unittest.main()
