# Copyright (C) 2002-2003 Juan David Ibez Palomar <jdavid@itaapy.com>
#               2003 Itaapy <contact@itaapy.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA


"""
This module contains several Python functions that get a dom node as
first parameter and perform some transformation on it. The dom node
must be from an extended tree (see the tree module in this package).

We call these methods rules, as they are somewhat equivalent to the
XSLT rules.
"""

__revision__ = "$Id: rules.py,v 1.2 2003/08/05 09:11:26 jdavid Exp $"



def rule_identity(node):
    """
    Rule that does not change the node.
    """
    # XXX Probably should be removed.
    return node


def rule_content(node, content):
    """
    Rule that replaces the content of the node.
    """
    node.xpy_content(content)
    return node


def rule_attribute(node, name, value):
    """
    Rule that sets the specified attribute of the node.
    """
    node.setAttribute(name, value)
    return node


def rule_remove(node):
    """
    Rule that removes the node.
    """
    parent = node.parentNode
    parent.removeChild(node)

    return None


def rule_repeat(node, rule, sequence):
    """
    Implements iteration over a sequence of objects, the rule is applied
    to a clone of the current node and receives as argument one element
    from the sequence.
    """
    # Get the parent
    parent = node.parentNode

    # Create and add the new nodes
    for x in sequence:
        # Create a clone
        clone = node.xpy_clone()

        # Insert it in the tree
        parent.insertBefore(clone, node)

        # Apply the rule to the clone
        clone = apply(rule, (clone, x))

    # Remove the current node
    parent.removeChild(node)

    return None


